#ifndef __BRIKXH__
#define __BRIKXH__
#include "SDL/SDL.h"

#define VERSION         "0.2.2"
#define WIDTH           640
#define HEIGHT          480
#define BPP             32
#define NAME_MAXLEN     16
#define MAX_HIGHENTRIES 20
#define MAX_TILES       99
#define TILE_WIDTH      60
#define TILE_HEIGHT     60
#define WINDOWX         0
#define WINDOWY         0
#define TILESET_DEFAULT "marble"
#define DELAY           20
#define TILE_EMPTY      0
#define TILE_WALL       1
#define TILE_RIGHT      2
#define TILE_LEFT       3
#define TILE_PEAK       4
#define TILE_PRIORRIGHT 5
#define TILE_PRIORLEFT  6
#define TILE_STRONG     7
#define TILE_STRONGRANDOM 10
#define TILE_STRONGPRIORRIGHT 8
#define TILE_STRONGPRIORLEFT 9
#define TILE_LEVITATOR  11
#define TILE_FIRST	    40
#define IsTile(a)       ((a)>=TILE_FIRST)
#define IsEmpty(a)      ((a)==TILE_EMPTY)
#define IsWeak(a)       ((a)==TILE_WALL || (a)==TILE_RIGHT || (a)==TILE_LEFT || (a)==TILE_PEAK || (a)==TILE_PRIORRIGHT || (a)==TILE_PRIORLEFT)

#define DIR_DATA        "data/"
#define DIR_BACKGROUND  DIR_DATA"backgrounds/"
#define DIR_LEVEL       DIR_DATA"levels/"
#define DIR_SAVE        DIR_DATA"saves/"
#define DIR_IMAGE       DIR_DATA"images/"
#define DIR_MUSIC       DIR_DATA"music/"
#define DIR_SOUND       DIR_DATA"sounds/"
#define DIR_TILESET     DIR_DATA"tilesets/"
#define DIR_FONT        DIR_DATA"fonts/"
#define DIR_HELP        DIR_DATA"help/"
#define FILE_HIGH       DIR_DATA"high.dat"
#define FONTNAME_SCORE  DIR_FONT"gas.ttf"
#define FONTNAME_NORMAL DIR_FONT"bankgothic.ttf"
#define FONTSIZE_SCORE  30
#define FONTSIZE_SMALL  16

#define BACKGROUND_MENU DIR_BACKGROUND"title_background.png"
#define BACKGROUND_BASE DIR_BACKGROUND"play.png"
#define BACKGROUND_EDIT DIR_BACKGROUND"edit.png"
#define IMAGE_QUIT      DIR_IMAGE"menu_quit.png"
#define IMAGE_PLAY      DIR_IMAGE"menu_play.png"
#define IMAGE_CONTINUE  DIR_IMAGE"menu_continue.png"
#define IMAGE_EDIT      DIR_IMAGE"menu_edit.png"
#define IMAGE_YES       DIR_IMAGE"menu_yes.png"
#define IMAGE_NO        DIR_IMAGE"menu_no.png"
#define IMAGE_HELP      DIR_IMAGE"menu_help.png"
#define IMAGE_HIGHS     DIR_IMAGE"menu_highs.png"
#define IMAGE_ASKQUIT   DIR_IMAGE"ask_quit.png"
#define IMAGE_OPTIONS   DIR_IMAGE"menu_options.png"
#define IMAGE_LOGO      DIR_IMAGE"title_brix.png"
#define IMAGE_BUILD     DIR_IMAGE"button_build.png"
#define IMAGE_CLEAR     DIR_IMAGE"button_clear.png"
#define IMAGE_NORMAL    DIR_IMAGE"button_normal.png"
#define IMAGE_UNDO      DIR_IMAGE"button_undo.png"
#define IMAGE_SAVEBUTTON DIR_IMAGE"button_save.png"
#define IMAGE_FULLSCREEN DIR_IMAGE"menu_fullscreen.png"
#define IMAGE_TILESET   DIR_IMAGE"menu_tileset.png"
#define IMAGE_BACK      DIR_IMAGE"menu_back.png"
#define IMAGE_LOAD	    DIR_IMAGE"load.png"
#define IMAGE_SAVE	    DIR_IMAGE"save.png"
#define IMAGE_CLOCK	    DIR_IMAGE"clock.png"
#define IMAGE_TIMER	    DIR_IMAGE"timer.png"
#define IMAGE_SAND 	    DIR_IMAGE"sand.png"
#define IMAGE_RELOAD	DIR_IMAGE"reload.png"

#define ALPHA1	80
#define ALPHA2  160

struct high_entry
{
    long score;
    char name[NAME_MAXLEN+1];
};

typedef high_entry high[MAX_HIGHENTRIES];
typedef int level[8][8];

enum modes {MODE_NORMAL,MODE_BUILD,MODE_CLEAR,MODE_UNDO,MODE_RELOAD,MODE_SAVE,MODE_END};
const int cost[]={    0,       100,       200,      200,        500,      500,       0};

SDL_Surface *LoadImage(char *fname,int Rkey=-1,int Gkey=-1,int Bkey=-1);
void PutSurface(SDL_Surface *src,SDL_Surface *dest,SDL_Rect pos,int clipx=-1,int clipy=-1,int width=-1,int height=-1);
void PutSurface(SDL_Surface *src,SDL_Surface *dest,int x=0,int y=0,int clipx=-1,int clipy=-1,int width=-1,int height=-1);
int LoadLevel(char *fname,level &t);
int SaveLevel(char *fname,level t,int timelimit);
int Input(SDL_Surface *screen,TTF_Font *font,SDL_Color color,int x,int y,char *s,int maxlen);
int SaveHigh(long score,char *name);
int SaveGame(level table,long score,int levnum,long timer,int timelimit,char *name="0");
int LoadGame(level &table,long &score,int &levnum,long &timer,int &timelimit,char *name="0");
#endif
