#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "SDL/SDL.h"
#include "SDL/SDL_image.h"
#include "SDL/SDL_ttf.h"
#include "brikx.h"
#include "button.h"

char tileset [256];
bool fullscreen=false;
int GRAVITY=100;
SDL_Surface *screen=NULL;

void Help(long &timer,char *type="play")
{
    SDL_Surface *help=NULL,*back=NULL,*save=NULL;
    bool quit=false,next;
    int page=0;
    char s[1024];
    long t0=SDL_GetTicks();
    Uint32 rmask,gmask,bmask,amask;
    SDL_Event event;
       #if SDL_BYTEORDER == SDL_BIG_ENDIAN
           rmask = 0xff000000;
           gmask = 0x00ff0000;
           bmask = 0x0000ff00;
           amask = 0x000000ff;
       #else
           rmask = 0x000000ff;
           gmask = 0x0000ff00;
           bmask = 0x00ff0000;
           amask = 0xff000000;
       #endif
    if (!(back=SDL_CreateRGBSurface(SDL_SRCALPHA | SDL_SWSURFACE | SDL_RLEACCEL,WIDTH-80,HEIGHT-80,BPP,rmask,gmask,bmask,amask))) return;
    save=SDL_CreateRGBSurface(SDL_SRCALPHA | SDL_SWSURFACE | SDL_RLEACCEL,WIDTH,HEIGHT,BPP,rmask,gmask,bmask,amask);
    if (save)
      {
        SDL_LockSurface(save);
        SDL_LockSurface(screen);
        memcpy(save->pixels,screen->pixels,WIDTH*HEIGHT*BPP/8);
        SDL_UnlockSurface(save);
        SDL_UnlockSurface(screen);
      }
    SDL_FillRect(back,NULL,100*amask);
    while (!quit)
      {
        sprintf(s,"%s%s/%d.png",DIR_HELP,type,page++);
        if (help) SDL_FreeSurface(help);
        if (!(help=LoadImage(s,0,0,0)))

          {
            quit=next=true;
            fprintf(stderr,"Error loading help image %s\n",s);
          }
        else
          {
            if (save)
              {
                SDL_LockSurface(save);
                SDL_LockSurface(screen);
                memcpy(screen->pixels,save->pixels,WIDTH*HEIGHT*BPP/8);
                SDL_UnlockSurface(save);
                SDL_UnlockSurface(screen);
              }
            PutSurface(back,screen,40,40);
            PutSurface(help,screen);
            SDL_Flip(screen);
            next=false;
          }
        while (!next)
          {
            SDL_Delay(DELAY);
            while (SDL_PollEvent(&event))
            {
              switch (event.type)
                {
                   case SDL_QUIT:
                     quit=next=true;
                   break;
                   case SDL_MOUSEBUTTONDOWN:
                        if (event.button.button==SDL_BUTTON_MIDDLE)page-=2;
                        next=true;
                   break;
                   case SDL_KEYDOWN:
                     switch (event.key.keysym.sym)
                       {
                         case SDLK_PAGEUP:
                         case SDLK_BACKSPACE:
                         case SDLK_LEFT:
                         case SDLK_MINUS:
                                  page-=2;
                                  next=true;
                                break;
                         case SDLK_ESCAPE:
                         case SDLK_q:
                         case SDLK_x:
                         case SDLK_F10:
                                quit=next=true;
                              break;
                         default:next=true;
                       }
                   break;
            }
          }
       }
      }
    if (timer>=0) timer+=SDL_GetTicks()-t0;
    if (save) SDL_FreeSurface(save);
    SDL_FreeSurface(back);
    if (help) SDL_FreeSurface(help);
}

bool Escape(long &timer)
{
     Button *yes=new Button(270,200,100,40,IMAGE_YES),
             *no=new Button(270,280,100,40,IMAGE_NO);
     SDL_Surface *text=LoadImage(IMAGE_ASKQUIT),*back=NULL;
     SDL_Event event;
     Uint32 rmask,gmask,bmask,amask;
       #if SDL_BYTEORDER == SDL_BIG_ENDIAN
           rmask = 0xff000000;
           gmask = 0x00ff0000;
           bmask = 0x0000ff00;
           amask = 0x000000ff;
       #else
           rmask = 0x000000ff;
           gmask = 0x0000ff00;
           bmask = 0x00ff0000;
           amask = 0xff000000;
       #endif
     if (!(back=SDL_CreateRGBSurface(SDL_SRCALPHA | SDL_SWSURFACE | SDL_RLEACCEL,WIDTH,HEIGHT,BPP,rmask,gmask,bmask,amask))) return true;
     SDL_LockSurface(back);
     SDL_LockSurface(screen);
     memcpy(back->pixels,screen->pixels,WIDTH*HEIGHT*BPP/8);
     SDL_UnlockSurface(back);
     SDL_UnlockSurface(screen);
     long t0=SDL_GetTicks();
     int quit=0;
     while (!quit)
       {
         SDL_FillRect(screen,NULL,1);
         PutSurface(text,screen,270,120);
         yes->Show(screen,yes->Inside(event)*127+128);
         no->Show(screen,no->Inside(event)*127+128);
         SDL_Flip(screen);
         while (SDL_PollEvent(&event))
           switch (event.type)
             {
               case SDL_MOUSEBUTTONDOWN:
                      if (yes->Inside(event)) quit=2;
                      if (no->Inside(event)) quit=1;
                    break;
               case SDL_KEYDOWN:
                      switch (event.key.keysym.sym)
                         {
                          case SDLK_ESCAPE:
                          case SDLK_n:
                          case SDLK_p:
                                 quit=1;
                               break;
                          case SDLK_RETURN:
                          case SDLK_z:
                          case SDLK_y:
                                 quit=2;
                               break;
                         }
                   break;
             }
       }
     SDL_LockSurface(back);
     SDL_LockSurface(screen);
     memcpy(screen->pixels,back->pixels,WIDTH*HEIGHT*BPP/8);
     SDL_UnlockSurface(back);
     SDL_UnlockSurface(screen);
     SDL_FreeSurface(text);
     SDL_FreeSurface(back);
     delete yes;
     delete no;
     if (timer>=0) timer+=SDL_GetTicks()-t0;
     return quit==2;
}

void PutTile(SDL_Surface *tile,int x,int y)
{
    PutSurface(tile,screen,WINDOWX+x*TILE_WIDTH,WINDOWY+y*TILE_HEIGHT);
}

void PutEmpty(int x,int y)
{
    SDL_Rect p;
    p.x=WINDOWX+x*TILE_WIDTH;
    p.y=WINDOWY+y*TILE_HEIGHT;
    p.w=TILE_WIDTH;
    p.h=TILE_HEIGHT;
    SDL_FillRect(screen,&p,0);
}

void DropLeft(SDL_Surface *tile,level &table,level drop,int j,int i)
{
     if (j && IsEmpty(table[j-1][i+1]) && !drop[j-1][i])
           {
             table[j-1][i+1]=table[j][i];
             table[j][i]=TILE_EMPTY;
             PutTile(tile,j-1,i+1);
             PutEmpty(j,i);
           }
}

void DropRight(SDL_Surface *tile,level &table,level drop,int j,int i)
{
     if (j<7 && IsEmpty(table[j+1][i+1]) && !drop[j+1][i])
           {
             table[j+1][i+1]=table[j][i];
             table[j][i]=TILE_EMPTY;
             PutTile(tile,j+1,i+1);
             PutEmpty(j,i);
           }
}

void Play(char *load="",int playlevel=0)
{
    bool quit,endgame=false,undoable,loadgame=*load;
    float time=0;
    int i,j,k,levnum=1,timelimit,mode=MODE_NORMAL,count;
    long timer,nulltime,step,paused,score=0;
    SDL_Rect pos;
    char s[1024];
    Button *button_build=new Button(500,260,60,60,IMAGE_BUILD),*button_clear=new Button(560,260,60,60,IMAGE_CLEAR),
	   *button_undo=new Button(500,330,60,60,IMAGE_UNDO),*button_normal=new Button(500,400,60,60,IMAGE_NORMAL),
	   *button_reload=new Button(560,330,60,60,IMAGE_RELOAD),*button_save=new Button(560,400,60,60,IMAGE_SAVEBUTTON);
    level table,drop,del,undo,levity;
    SDL_Color color={200,255,220};
    SDL_Event event;
    TTF_Font *font=TTF_OpenFont(FONTNAME_SCORE,FONTSIZE_SCORE),*smallfont=TTF_OpenFont(FONTNAME_SCORE,FONTSIZE_SMALL);
    SDL_Surface *base=LoadImage(BACKGROUND_BASE),*tile[MAX_TILES],
		*clock=LoadImage(IMAGE_TIMER,0,0,0),*sand=LoadImage(IMAGE_SAND),
                *text=NULL;
     Uint32 rmask,gmask,bmask,amask;
       #if SDL_BYTEORDER == SDL_BIG_ENDIAN
           rmask = 0xff000000;
           gmask = 0x00ff0000;
           bmask = 0x0000ff00;
           amask = 0x000000ff;
       #else
           rmask = 0x000000ff;
           gmask = 0x0000ff00;
           bmask = 0x00ff0000;
           amask = 0xff000000;
       #endif
    if (!font || !smallfont)
      {
        fprintf(stderr,"ERROR: Font file %s not found\n",FONTNAME_SCORE);
        return;
      }
    if (!base)
      {
        fprintf(stderr,"ERROR: Image file %s not found\n",BACKGROUND_BASE);
        return;
      }
    if (!clock)
      {
        fprintf(stderr,"ERROR: Image file %s not found\n",IMAGE_TIMER);
        return;
      }
    for (i=0; i<MAX_TILES; i++)
      {
	    sprintf(s,"%s%s/%d.png",DIR_TILESET,tileset,i);
	    if (!(tile[i]=LoadImage(s)))
	    {
	      sprintf(s,"%s%s/9%d.png",DIR_TILESET,TILESET_DEFAULT,i);
          if (!(tile[i]=LoadImage(s)))
            fprintf(stderr,"WARNING: %s%s/%d.png not found!\n",DIR_TILESET,tileset,i);
        }
      }
   while (!endgame)
   {
     if (playlevel)
       {
          levnum=playlevel;
          endgame=true;
          score=99999;
       }
    quit=false;
    sprintf(s,"%s%d.lvl",DIR_LEVEL,levnum);
    for (i=0; i<=7; i++)
      for (j=0; j<=7; j++)
        drop[j][i]=0;
    SDL_FillRect(screen,NULL,0);
    paused=0;
    timer=nulltime=SDL_GetTicks();
    if (time>1 || !(timelimit=LoadLevel(s,table)))
      {
	    fprintf(stderr,"ERROR: Level file %s not found!\n",s);
	    endgame=true;
        if (score)
	    {
          SDL_FillRect(screen,NULL,0);
	      sprintf(s,"End score: %ld",score);
	      text=TTF_RenderText_Solid(font,s,color);
	      PutSurface(text,screen,(WIDTH-text->w)/2,100);
	      strcpy(s,"Enter your name:");
	      Input(screen,font,color,10,100+2*FONTSIZE_SCORE,s,NAME_MAXLEN);
	      SaveHigh(score,s);
	    }
      }
    else
    {
     if (loadgame)
       {
         LoadGame(table,score,levnum,timer,timelimit,load);
	 nulltime-=timer;
         loadgame=false;
       }
	 for (i=0; i<=7; i++)
	   for (j=0; j<=7; j++)
	     {
           PutTile(tile[table[j][i]],j,i);
           del[j][i]=levity[j][i]=0;
           undo[j][i]=table[j][i];
           for (k=i; k<=7; k++)
             levity[j][i]=levity[j][i] | (table[j][k]==TILE_LEVITATOR);
         }
     undoable=false;
     SDL_SetAlpha(clock,SDL_RLEACCEL | SDL_SRCALPHA,200);
     while (!quit)
       {
         SDL_Delay(DELAY);
         step=SDL_GetTicks()-timer;
         timer=SDL_GetTicks();
         time=(timer-nulltime-paused)/1000.0/timelimit;
         sprintf(s,"%ld",score);
         if (text) SDL_FreeSurface(text);
         text=TTF_RenderText_Solid(font,s,color);
	 PutSurface(base,screen,480,0);
	 pos.h=round(sand->h/2.0*(1-time));
	 pos.y=sand->h/2-pos.h;
	 PutSurface(text,screen,618-text->w,190);
	 PutSurface(sand,screen,504,22+pos.y,0,pos.y,sand->w,round(sand->h/2*(1-time)));
	 PutSurface(sand,screen,504,22+sand->h-pos.y,0,sand->h-pos.y,sand->w,round(sand->h/2*time));
	 PutSurface(clock,screen,499,16);
	 button_build->Show(screen,mode==MODE_BUILD?255:score>=cost[MODE_BUILD]?ALPHA2:ALPHA1);
	 button_clear->Show(screen,mode==MODE_CLEAR?255:score>=cost[MODE_CLEAR]?ALPHA2:ALPHA1);
	 button_undo->Show(screen,(undoable && score>=cost[MODE_UNDO])?ALPHA2:ALPHA1);
	 button_save->Show(screen,(score>=cost[MODE_SAVE])?ALPHA2:ALPHA1);
	 button_reload->Show(screen,mode==MODE_RELOAD?255:score>=cost[MODE_RELOAD]?ALPHA2:ALPHA1);
	 button_normal->Show(screen,mode==MODE_NORMAL?255:ALPHA2);
	 sprintf(s,"%d",cost[MODE_BUILD]);
     if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(smallfont,s,color);
	 SDL_SetAlpha(text,SDL_SRCALPHA | SDL_RLEACCEL,mode==MODE_BUILD?255:score>=cost[MODE_BUILD]?ALPHA2:ALPHA1);
	 PutSurface(text,screen,515,308);
	 sprintf(s,"%d",cost[MODE_CLEAR]);
     if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(smallfont,s,color);
	 SDL_SetAlpha(text,SDL_SRCALPHA | SDL_RLEACCEL,mode==MODE_CLEAR?255:score>=cost[MODE_CLEAR]?ALPHA2:ALPHA1);
	 PutSurface(text,screen,575,308);
	 sprintf(s,"%d",cost[MODE_UNDO]);
     if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(smallfont,s,color);
	 SDL_SetAlpha(text,SDL_SRCALPHA | SDL_RLEACCEL,(undoable && score>=cost[MODE_UNDO])?ALPHA2:ALPHA1);
	 PutSurface(text,screen,510,378);
	 sprintf(s,"%d",cost[MODE_RELOAD]);
     if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(smallfont,s,color);
	 SDL_SetAlpha(text,SDL_SRCALPHA | SDL_RLEACCEL,mode==MODE_RELOAD?255:score>=cost[MODE_RELOAD]?ALPHA2:ALPHA1);
	 PutSurface(text,screen,575,378);
	 sprintf(s,"%d",cost[MODE_SAVE]);
     if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(smallfont,s,color);
	 SDL_SetAlpha(text,SDL_SRCALPHA | SDL_RLEACCEL,score>=cost[MODE_SAVE]?ALPHA2:ALPHA1);
	 PutSurface(text,screen,575,448);
	 sprintf(s,"%d",levnum);
     if (text) SDL_FreeSurface(text);
	 text=TTF_RenderText_Solid(font,s,color);
	 PutSurface(text,screen,618-text->w,20);
     for (count=i=0; i<=7; i++)
       for (j=0; j<=7; j++)
         if (IsTile(table[j][i]))
         {
           count++;
           if (levity[j][i] && i && IsEmpty(table[j][i-1]))
             {
                table[j][i-1]=table[j][i];
                table[j][i]=TILE_EMPTY;
                PutTile(tile[table[j][i-1]],j,i-1);
                PutEmpty(j,i);
                drop[j][i-1]=0;
             }
            else
           if (i<7 && !drop[j][i] && !levity[j][i])
             switch (table[j][i+1])
               {
                  case TILE_EMPTY:
                         drop[j][i]=GRAVITY;
                       break;
                  case TILE_LEFT:
                         DropLeft(tile[table[j][i]],table,drop,j,i);
                       break;
                  case TILE_RIGHT:
                         DropRight(tile[table[j][i]],table,drop,j,i);
                        break;
                  case TILE_PEAK:
                  case TILE_STRONGRANDOM:
                          if (rand()%2) DropLeft(tile[table[j][i]],table,drop,j,i);
                          else DropRight(tile[table[j][i]],table,drop,j,i);
                        break;
                  case TILE_STRONGPRIORLEFT:
                  case TILE_PRIORLEFT:
                          if (i<7)
                            {
                              if (j && IsEmpty(table[j-1][i+1])) DropLeft(tile[table[j][i]],table,drop,j,i);
                              else if (j<7 && IsEmpty(table[j+1][i+1])) DropRight(tile[table[j][i]],table,drop,j,i);
                            }
                        break;
                  case TILE_STRONGPRIORRIGHT:
                  case TILE_PRIORRIGHT:
                          if (i<7)
                            {
                                if (j<7 && IsEmpty(table[j+1][i+1])) DropRight(tile[table[j][i]],table,drop,j,i);
                                else if (j && IsEmpty(table[j-1][i+1])) DropLeft(tile[table[j][i]],table,drop,j,i);
                            }
                        break;
               }
           if (drop[j][i])
             {
               if ((drop[j][i]-=step)<=0)
                 {
                   drop[j][i]=0;
                   table[j][i+1]=table[j][i];
                   table[j][i]=TILE_EMPTY;
                   PutEmpty(j,i);
                   PutTile(tile[table[j][i+1]],j,i+1);
                 }
               else
                 {
                    PutEmpty(j,i);    
                    PutSurface(tile[table[j][i]],screen,j*TILE_WIDTH+WINDOWX,round((i+1-drop[j][i]/float(GRAVITY))*TILE_HEIGHT+WINDOWY));
                 }
             }
          }
     for (i=0; i<=7; i++)
       for (j=0; j<=7; j++)
          if (IsTile(table[j][i]) && !drop[j][i] && (
           (j && !drop[j-1][i] && table[j-1][i]==table[j][i]) ||
           (i && !drop[j][i-1] && table[j][i-1]==table[j][i]) ||
           (j<7 && !drop[j+1][i] && table[j+1][i]==table[j][i]) ||
           (i<7 && !drop[j][i+1] && table[j][i+1]==table[j][i])
           )) del[j][i]=1; else del[j][i]=0;
     for (i=0; i<=7; i++)
       for (j=0; j<=7; j++)
         if (del[j][i])
           {
              del[j][i]=0;
              table[j][i]=TILE_EMPTY;
              PutEmpty(j,i);
           }
     quit=!count || time>1;
	 SDL_Flip(screen);
	 while (SDL_PollEvent(&event))
	 {
	   switch (event.type)
		     {
		       case SDL_QUIT:
			          quit=endgame=Escape(paused);
			        break;
		       case SDL_KEYDOWN:
			      switch (event.key.keysym.sym)
				{
				  case ' ':
				         mode=MODE_NORMAL;
				       break;
				  case SDLK_f:
                        screen=SDL_SetVideoMode(WIDTH,HEIGHT,BPP,SDL_SWSURFACE | (fullscreen=!fullscreen)?SDL_FULLSCREEN:0);
                        for (i=0; i<=7; i++)
                          for (j=0; j<=7; j++)
                            PutTile(tile[table[j][i]],j,i);
                       break;
				  case SDLK_F1:
                  case SDLK_h:
                          SDL_FillRect(screen,NULL,1);
                          Help(paused,"play");
				  case SDLK_r:
                         for (i=0; i<=7; i++)
                           for (j=0; j<=7; j++)
                             PutTile(tile[table[j][i]],j,i);
                       break;
				  case SDLK_TAB:
				         while (score<cost[++mode] || mode==MODE_UNDO || mode==MODE_SAVE || mode==MODE_RELOAD)
					      ;
					 if (mode==MODE_END) mode=0;
				       break;
				  case SDLK_p:
				  case SDLK_ESCAPE:
				  case SDLK_q:
				  case SDLK_F10:
					     quit=endgame=Escape(paused);
				       break;
				  case SDLK_F12:
				         if (SDL_GetModState() & KMOD_RCTRL)
					     for (i=0; i<=7; i++)
						   for (j=0; j<=7; j++)
						     table[j][i]=TILE_EMPTY;
				       break;
				}
			    break;
			case SDL_MOUSEBUTTONDOWN:
				j=(event.button.x-WINDOWX)/TILE_WIDTH;
				i=(event.button.y-WINDOWY)/TILE_HEIGHT;
			        if (button_build->Inside(event) && score>=cost[MODE_BUILD]) mode=MODE_BUILD;
				else if (button_clear->Inside(event) && score>=cost[MODE_CLEAR]) mode=MODE_CLEAR;
				else if (button_save->Inside(event) && score>=cost[MODE_SAVE])
				  {
                    if (!SaveGame(table,score-cost[MODE_SAVE],levnum,step=timer-nulltime,timelimit))
                       score-=cost[MODE_SAVE];
                  }
				else if (button_normal->Inside(event)) mode=MODE_NORMAL;
				else if (button_undo->Inside(event) && score>=cost[MODE_UNDO]) 
                  {
                     undoable=false;
                     score-=cost[MODE_UNDO];
                     for (i=0; i<=7; i++)
                       for (j=0; j<=7; j++)
                        PutTile(tile[table[j][i]=undo[j][i]],j,i);
                  }
				else if (button_reload->Inside(event) && score>=cost[MODE_RELOAD])
				  {
				    quit=true;
				    score-=cost[MODE_RELOAD];
				  }
				else if (event.button.button==SDL_BUTTON_MIDDLE) mode=MODE_NORMAL;
				else if (j<=7 && i<=7)
				  switch (mode)
				  {
				    case MODE_NORMAL:
					switch (event.button.button)
					  {
					    case SDL_BUTTON_LEFT:
						   if (IsTile(table[j][i]) && j && IsEmpty(table[j-1][i]))
						     {
						       undoable=true;
						       memcpy(undo,table,sizeof(level));
						       table[j-1][i]=table[j][i];
						       table[j][i]=TILE_EMPTY;
						       PutEmpty(j,i);
						       PutTile(tile[table[j-1][i]],j-1,i);
						     }
						 break;
					    case SDL_BUTTON_RIGHT:
						   if (IsTile(table[j][i]) && j<7 && IsEmpty(table[j+1][i]))
						     {
						       undoable=true;
						       memcpy(undo,table,sizeof(level));
						       table[j+1][i]=table[j][i];
						       table[j][i]=TILE_EMPTY;
						       PutEmpty(j,i);
						       PutTile(tile[table[j+1][i]],j+1,i);
						     }
						 break;
					  }
					break;
				    case MODE_BUILD:
					   if (IsEmpty(table[j][i]) && score>=cost[MODE_BUILD])
					     {
					       score-=cost[MODE_BUILD];
					       table[j][i]=TILE_WALL;
					       PutTile(tile[TILE_WALL],j,i);
					     }
					break;
				    case MODE_CLEAR:
					   if (IsWeak(table[j][i]) && score>=cost[MODE_CLEAR])
					     {
					       score-=cost[MODE_CLEAR];
					       table[j][i]=TILE_EMPTY;
					       PutEmpty(j,i);
					     }
					break;
				  }
			     break;
		     }
	       }
	    }
    if (time<=1 && !count) score+=round((1-time)*levnum*100)+levnum*10;
    if (!count) levnum++;
    }
   }
    SDL_FreeSurface(base);
    SDL_FreeSurface(text);
    SDL_FreeSurface(clock);
    SDL_FreeSurface(sand);
    TTF_CloseFont(font);
    TTF_CloseFont(smallfont);
    delete button_build;
    delete button_clear;
    delete button_undo;
    delete button_save;
    delete button_normal;
    delete button_reload;
    for (i=0; i<MAX_TILES; i++) if (tile[i]) SDL_FreeSurface(tile[i]);
}

void Options()
{
    Button *full=new Button(WIDTH/2-100-79,(HEIGHT-200)/4*0+30,158,40,IMAGE_FULLSCREEN),
           *tile=new Button(WIDTH/2-100-79,(HEIGHT-200)/4*1+30,158,40,IMAGE_TILESET),
           *b_back=new Button(WIDTH/2-100-79,(HEIGHT-200)/4*2+30,158,40,IMAGE_BACK);
    TTF_Font *font=NULL;
    SDL_Surface *text=NULL,*back=NULL,*save=NULL;
    SDL_Event event;
    SDL_Color color={100,200,255};
    font=TTF_OpenFont(FONTNAME_NORMAL,FONTSIZE_SMALL);
    if (!font) return;
    Uint32 rmask,gmask,bmask,amask;
       #if SDL_BYTEORDER == SDL_BIG_ENDIAN
           rmask = 0xff000000;
           gmask = 0x00ff0000;
           bmask = 0x0000ff00;
           amask = 0x000000ff;
       #else
           rmask = 0x000000ff;
           gmask = 0x0000ff00;
           bmask = 0x00ff0000;
           amask = 0xff000000;
       #endif
    if (!(save=SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_RLEACCEL,WIDTH,HEIGHT,BPP,rmask,gmask,bmask,amask))) return;
    if (!(back=SDL_CreateRGBSurface(SDL_SRCALPHA | SDL_SWSURFACE | SDL_RLEACCEL,WIDTH-200,HEIGHT-200,BPP,rmask,gmask,bmask,amask))) return;
    SDL_FillRect(back,NULL,100*amask);
    bool quit=false;
    srand(time(NULL));
    SDL_LockSurface(screen);
    SDL_LockSurface(save);
    memcpy(save->pixels,screen->pixels,WIDTH*HEIGHT*BPP/8);
    SDL_UnlockSurface(screen);
    SDL_UnlockSurface(save);
    full->Show(back);
    //tile->Show(back);
    b_back->Show(back);
    PutSurface(back,screen,(screen->w-back->w)/2,(screen->h-back->h)/2);
    SDL_Flip(screen);
    while (!quit)
      {
        SDL_Delay(DELAY);
        while (SDL_PollEvent(&event))
        {
          switch (event.type)
            {
              case SDL_QUIT:
                     quit=true;
                   break;
              case SDL_MOUSEBUTTONDOWN:
                     if (full->Inside(event.button.x-100,event.button.y-100))
                       {
                          screen=SDL_SetVideoMode(WIDTH,HEIGHT,BPP,SDL_SWSURFACE | (fullscreen=!fullscreen)?SDL_FULLSCREEN:0);
                          SDL_LockSurface(screen);
                          SDL_LockSurface(save);
                          memcpy(screen->pixels,save->pixels,WIDTH*HEIGHT*BPP/8);
                          SDL_UnlockSurface(screen);
                          SDL_UnlockSurface(save);
                          PutSurface(back,screen,(screen->w-back->w)/2,(screen->h-back->h)/2);
                          SDL_Flip(screen);
                       }
                     if (tile->Inside(event.button.x-100,event.button.y-100)) ;
                     if (b_back->Inside(event.button.x-100,event.button.y-100)) quit=true;
                   break;
              case SDL_KEYDOWN:
                     switch (event.key.keysym.sym)
                       {
         				 case SDLK_f:
                                screen=SDL_SetVideoMode(WIDTH,HEIGHT,BPP,SDL_SWSURFACE | (fullscreen=!fullscreen)?SDL_FULLSCREEN:0);
                                SDL_LockSurface(screen);
                                SDL_LockSurface(save);
                                memcpy(screen->pixels,save->pixels,WIDTH*HEIGHT*BPP/8);
                                SDL_UnlockSurface(screen);
                                SDL_UnlockSurface(save);                                
                                PutSurface(back,screen,(screen->w-back->w)/2,(screen->h-back->h)/2);
                                SDL_Flip(screen);
                              break;
                         case SDLK_ESCAPE:
                         case SDLK_q:
                         case SDLK_x:
                         case SDLK_F10:
                                quit=true;
                              break;
                       }
                   break;
            }
          }
      }
    if (font) TTF_CloseFont(font);
    if (text) SDL_FreeSurface(text);
    if (back) SDL_FreeSurface(back);
    if (save) SDL_FreeSurface(save);
    delete full;
    delete tile;
}

void Edit()
{
    bool quit=false;
    long i;
    int j,levnum=1,timelimit,active=TILE_FIRST;
    char s[1024],t[1024];
    level table;
    SDL_Color color={200,255,220};
    SDL_Event event;
    Uint32 rmask,gmask,bmask,amask;
       #if SDL_BYTEORDER == SDL_BIG_ENDIAN
           rmask = 0xff000000;
           gmask = 0x00ff0000;
           bmask = 0x0000ff00;
           amask = 0x000000ff;
       #else
           rmask = 0x000000ff;
           gmask = 0x0000ff00;
           bmask = 0x00ff0000;
           amask = 0xff000000;
       #endif    
    TTF_Font *font=TTF_OpenFont(FONTNAME_NORMAL,FONTSIZE_SCORE),*smallfont=TTF_OpenFont(FONTNAME_NORMAL,FONTSIZE_SMALL);
    SDL_Surface *base=LoadImage(BACKGROUND_EDIT),*tile[MAX_TILES],*tile_small[MAX_TILES],*text=NULL;
    Button *load=new Button(WINDOWX+TILE_WIDTH*8+TILE_WIDTH/2,WINDOWY+TILE_HEIGHT*7,32,32,IMAGE_LOAD),*save=new Button(WINDOWX+TILE_WIDTH*9+4,WINDOWY+TILE_HEIGHT*7+4,24,24,IMAGE_SAVE),
           *clock=new Button(WINDOWX+TILE_WIDTH*9+TILE_WIDTH/2,WINDOWY+TILE_HEIGHT*7,32,32,IMAGE_CLOCK);
    if (!font || !smallfont)
      {
        fprintf(stderr,"ERROR: Font file %s not found\n",FONTNAME_SCORE);
        return;
      }
    if (!base)
      {
        fprintf(stderr,"ERROR: Image file %s not found\n",BACKGROUND_BASE);
        return;
      }
    for (i=0; i<MAX_TILES; i++)
      {
	    sprintf(s,"%s%s/%d.png",DIR_TILESET,tileset,i);
	    if (!(tile[i]=LoadImage(s,0,0,0)))
	      {
   	        sprintf(s,"%s%s/%d.png",DIR_TILESET,TILESET_DEFAULT,i);
	        if (!(tile[i]=LoadImage(s,0,0,0)))
              fprintf(stderr,"WARNING: %s%s/%d.png not found!\n",DIR_TILESET,tileset,i);
          }
	    sprintf(s,"%s%s/small/%d.png",DIR_TILESET,tileset,i);
	    if (!(tile_small[i]=LoadImage(s,0,0,0)))
	      {
   	        sprintf(s,"%s%s/small/%d.png",DIR_TILESET,TILESET_DEFAULT,i);
	        if (!(tile_small[i]=LoadImage(s,0,0,0)))
             fprintf(stderr,"WARNING: %s%s/small/%d.png not found!\n",DIR_TILESET,tileset,i);
          }
      }
    SDL_FillRect(screen,NULL,1);
    sprintf(s,"%s%d.lvl",DIR_LEVEL,levnum);
    if ((timelimit=LoadLevel(s,table))>=0)
    while (!quit)
    {
        SDL_Delay(DELAY);
        PutSurface(base,screen);
	for (i=0; i<=3; i++)
	  for (j=0; j<=8; j++)
	    if (tile_small[TILE_FIRST+i*9+j])
	    {
	      SDL_SetAlpha(tile_small[TILE_FIRST+i*9+j],SDL_SRCALPHA | SDL_RLEACCEL,TILE_FIRST+i*9+j==active?255:ALPHA2);
	      PutSurface(tile_small[TILE_FIRST+i*9+j],screen,WINDOWX+TILE_WIDTH*8+TILE_WIDTH*(i+1)/2,WINDOWY+TILE_HEIGHT*(j+1)/2);
	    }
	for (i=0; i<=2; i++)
	  for (j=0; j<=3;j++)
	    if (tile_small[i*4+j+1])
	    {
	      SDL_SetAlpha(tile_small[i*4+j+1],SDL_SRCALPHA | SDL_RLEACCEL,i*4+j+1==active?255:ALPHA2);
	      PutSurface(tile_small[i*4+j+1],screen,WINDOWX+TILE_WIDTH*8+TILE_WIDTH*(i+1)/2,WINDOWY+TILE_HEIGHT*(j+10)/2);
	    }
	for (i=0; i<=7; i++)
	  for (j=0; j<=7; j++)
	      PutSurface(tile[table[j][i]],screen,j*TILE_WIDTH+WINDOWX,i*TILE_HEIGHT+WINDOWY);
	load->Show(screen);
	save->Show(screen);
	clock->Show(screen);
	SDL_Flip(screen);
        while (SDL_PollEvent(&event))
        {
          switch (event.type)
            {
              case SDL_QUIT:
                     quit=Escape(i);
                   break;
              case SDL_MOUSEBUTTONDOWN:
                     if (load->Inside(event))
                       {
                         sprintf(s,"Load level (current:%d):",levnum);
                         if (Input(screen,font,color,10,200,s,100) && atoi(s)>0)
                           {
				             levnum=atoi(s);
             				 sprintf(t,"%s%s.lvl",DIR_LEVEL,s);
				             fprintf(stderr,"Loading: %s\n",t);
				             if (!(timelimit=LoadLevel(t,table)))
				             fprintf(stderr,"Error loading level %s",t);
                           }
                       }
                     else if (save->Inside(event)) 
                       {
                         sprintf(s,"Save as level number (current:%d):",levnum);
                         if (Input(screen,font,color,10,200,s,100) && atoi(s)>0)
                           {
                             levnum=atoi(s);
                             sprintf(t,"%s%s.lvl",DIR_LEVEL,s);
                             SaveLevel(t,table,timelimit);
                           }
                       }
                     else if (clock->Inside(event))
                       {
                         sprintf(s,"Enter timelimit (current:%d):",timelimit);
                         if (Input(screen,font,color,10,200,s,100) && atoi(s)>0) timelimit=atoi(s);
                       }
                     else 
                     if (event.button.x<TILE_WIDTH*8+WINDOWX)
                       {
                         j=(event.button.x-WINDOWX)/TILE_WIDTH;
		                 i=(event.button.y-WINDOWY)/TILE_HEIGHT;
		                 switch (event.button.button)
		                   {
		                     case SDL_BUTTON_LEFT:
		                            table[j][i]=active;
		                     break;
		                     case SDL_BUTTON_RIGHT:
		                            table[j][i]=TILE_EMPTY;
		                     break;
		           }
		       }
		     else
		       {
                         j=(event.button.x-TILE_WIDTH*8-WINDOWX-TILE_WIDTH/2)/(TILE_WIDTH/2);
                         i=(event.button.y-WINDOWY-TILE_HEIGHT/2)/(TILE_HEIGHT/2);
                         if (i<=8 && tile[TILE_FIRST+j*9+i]) active=TILE_FIRST+j*9+i;
                         else if (tile[(i-9)+j*4+1]) active=(i-9)+j*4+1;
		       }
                   break;
              case SDL_KEYDOWN:
                     switch (event.key.keysym.sym)
                       {
         				 case SDLK_f:
                                screen=SDL_SetVideoMode(WIDTH,HEIGHT,BPP,SDL_SWSURFACE | (fullscreen=!fullscreen)?SDL_FULLSCREEN:0);
                              break;
                         case SDLK_p:
                                Play("",levnum);
                              break;
                         case SDLK_c:
                                for (i=0; i<=7; i++)
                                  for (j=0; j<=7; j++)
                                    {
                                      table[j][i]=TILE_EMPTY;
                                      PutEmpty(j,i);
                                    }
                              break;
                         case SDLK_F1:
                         case SDLK_h:
                                Help(i,"edit");
                              break;
                         case SDLK_ESCAPE:
                         case SDLK_q:
                         case SDLK_x:
                         case SDLK_F10:
                                quit=Escape(i);
                              break;
                       }
                   break;
            }
          }
    }
    for (i=0; i<MAX_TILES; i++)
      {
        if (tile[i]) SDL_FreeSurface(tile[i]);
        if (tile_small[i]) SDL_FreeSurface(tile_small[i]);
      }
    TTF_CloseFont(font);
    delete load;
    delete save;
    delete clock;
    TTF_CloseFont(smallfont);
    SDL_FreeSurface(base);
    if (text) SDL_FreeSurface(text);
}

void Highs()
{
    TTF_Font *font=NULL;
    SDL_Surface *text=NULL,*back=NULL;
    SDL_Event event;
    SDL_Color color={100,200,255};
    font=TTF_OpenFont(FONTNAME_NORMAL,FONTSIZE_SMALL);
    if (!font) return;
    Uint32 rmask,gmask,bmask,amask;
       #if SDL_BYTEORDER == SDL_BIG_ENDIAN
           rmask = 0xff000000;
           gmask = 0x00ff0000;
           bmask = 0x0000ff00;
           amask = 0x000000ff;
       #else
           rmask = 0x000000ff;
           gmask = 0x0000ff00;
           bmask = 0x00ff0000;
           amask = 0xff000000;
       #endif
    if (!(back=SDL_CreateRGBSurface(SDL_SRCALPHA | SDL_SWSURFACE | SDL_RLEACCEL,FONTSIZE_SMALL*(NAME_MAXLEN+7),MAX_HIGHENTRIES*FONTSIZE_SMALL,BPP,rmask,gmask,bmask,amask))) return;
    SDL_FillRect(back,NULL,100*amask);
    bool quit=false;
    int i;
    char s[NAME_MAXLEN+1],s2[NAME_MAXLEN+1];
    FILE *f=fopen(FILE_HIGH,"r");
    if (!f) return;
    for (i=0; !feof(f); i++)
      {
        fgets(s,NAME_MAXLEN,f);
        fgets(s2,NAME_MAXLEN,f);
        if (s[strlen(s)-1]=='\n') s[strlen(s)-1]=0;
        if (s2[strlen(s2)-1]=='\n') s2[strlen(s2)-1]=0;
	if (*s2)
	{
	    if (text) SDL_FreeSurface(text);
	    text=TTF_RenderText_Solid(font,s2,color);
	    PutSurface(text,back,0,i*FONTSIZE_SMALL);
	    if (text) SDL_FreeSurface(text);
	    text=TTF_RenderText_Solid(font,s,color);
	    PutSurface(text,back,back->w-text->w,i*FONTSIZE_SMALL);
	}
      }
    fclose(f);
    PutSurface(back,screen,(screen->w-back->w)/2,(screen->h-back->h)/2);
    SDL_Flip(screen);
    while (!quit)
      {
        SDL_Delay(DELAY);
        while (SDL_PollEvent(&event))
        {
          switch (event.type)
            {
              case SDL_QUIT:
                     quit=true;
                   break;
              case SDL_MOUSEBUTTONDOWN:
                     quit=true;
                   break;
              case SDL_KEYDOWN:
                     switch (event.key.keysym.sym)
                       {
                         case SDLK_ESCAPE:
                         case SDLK_q:
                         case SDLK_x:
                         case SDLK_F10:
                                quit=true;
                              break;
                       }
                   break;
            }
          }
      }
    TTF_CloseFont(font);
    SDL_FreeSurface(text);
    SDL_FreeSurface(back);
}

int main(int argc,char *argv[])
{
    bool quit=false;
    long int temp;
    int i;
    SDL_Surface *menu=NULL,*logo=NULL,*text=NULL;
    SDL_Event   event;
    strcpy(tileset,TILESET_DEFAULT);
    if (argc>1)
      {
        for (i=1; i<argc; i++)
          {
            while (argv[i][0]=='-')
              strcpy(argv[i],argv[i]+1);
            if (!strcmp(argv[i],"help"))
              {
                puts("Brikx version "VERSION);
                puts("Known parameters:");
                puts("fullscreen     Start in fullscreen");
                puts("tileset=name   Uses tileset 'name'");
                puts("gravity=value  Power of gravity (default: 100)");
                return 0;
              } 
            if (!strcmp(argv[i],"fullscreen")) fullscreen=true;
            if (!strncmp(argv[i],"tileset=",8))
                printf("Using tileset %s\n",strcpy(tileset,argv[i]+8));
            if (!strncmp(argv[i],"gravity=",8) && atoi(argv[i]+8))
                printf("Gravity set to %d\n",GRAVITY=10000/atoi(argv[i]+8));
          }
      }
    if (SDL_Init(SDL_INIT_EVERYTHING)==-1 || TTF_Init())
      {
        fprintf(stderr,"You must have SDL, SDL_image and SDL_ttf packages installed!\n");
        return -1;
      }
    TTF_Font *font=NULL;
    font=TTF_OpenFont(FONTNAME_NORMAL,FONTSIZE_SMALL);
    SDL_Color color={200,255,255};
    screen=SDL_SetVideoMode(WIDTH,HEIGHT,BPP,SDL_SWSURFACE | (fullscreen?SDL_FULLSCREEN:0));
#define BUTTONSPACE  60
#define BUTTONTOP    30
    Button *button_play=new Button(450,BUTTONTOP+BUTTONSPACE*0,158,40,IMAGE_PLAY),
           *button_continue=new Button(450,BUTTONTOP+BUTTONSPACE*1,158,40,IMAGE_CONTINUE),
           *button_edit=new Button(450,BUTTONTOP+BUTTONSPACE*2,158,40,IMAGE_EDIT),
           *button_options=new Button(450,BUTTONTOP+BUTTONSPACE*3,158,40,IMAGE_OPTIONS),
           *button_help=new Button(450,BUTTONTOP+BUTTONSPACE*4,158,40,IMAGE_HELP),
           *button_highs=new Button(450,BUTTONTOP+BUTTONSPACE*5,158,40,IMAGE_HIGHS),
           *button_quit=new Button(450,BUTTONTOP+BUTTONSPACE*6,158,40,IMAGE_QUIT);
    if (!screen)
      {
        SDL_Quit();
        return -2;
      }
    SDL_WM_SetCaption("Brikx by glezmen",NULL);
    menu=LoadImage(BACKGROUND_MENU);
    logo=LoadImage(IMAGE_LOGO);
    SDL_SetAlpha(logo,SDL_SRCALPHA | SDL_RLEACCEL,192);
    if (font) text=TTF_RenderText_Solid(font,"Brikx by glezmen (bokorn@tvn.hu)     Version: "VERSION,color);
    while (!quit)
      {
        SDL_Delay(DELAY);
        SDL_FillRect(screen,NULL,0);
        PutSurface(menu,screen);
        PutSurface(logo,screen,80,40);
        button_quit->Show(screen,button_quit->Inside(event)?255:ALPHA2);
        button_options->Show(screen,button_options->Inside(event)?255:ALPHA2);
        button_play->Show(screen,button_play->Inside(event)?255:ALPHA2);
        button_help->Show(screen,button_help->Inside(event)?255:ALPHA2);
        button_continue->Show(screen,button_continue->Inside(event)?255:ALPHA2);
        button_highs->Show(screen,button_highs->Inside(event)?255:ALPHA2);
        button_edit->Show(screen,button_edit->Inside(event)?255:ALPHA2);
        if (text) PutSurface(text,screen,10,HEIGHT-FONTSIZE_SMALL*2);
        SDL_Flip(screen);
        while (SDL_PollEvent(&event))
          switch (event.type)
            {
              case SDL_QUIT:
                     quit=true;
                   break;
              case SDL_MOUSEBUTTONDOWN:
                     if (button_quit->Inside(event)) quit=true;
                     if (button_play->Inside(event)) Play();
                     if (button_options->Inside(event)) Options();
                     if (button_help->Inside(event)) Help(temp);
                     if (button_highs->Inside(event)) Highs();
                     if (button_continue->Inside(event)) Play("0");
                     if (button_edit->Inside(event)) Edit();
                   break;
              case SDL_KEYDOWN:
                     switch (event.key.keysym.sym)
                       {
                         case SDLK_o:
                                Options();
                              break;
                         case SDLK_p:
                         case SDLK_RETURN:
                                Play();
                              break;
                         case SDLK_h:
                                Help(temp);
                              break;
         				 case SDLK_f:
                                screen=SDL_SetVideoMode(WIDTH,HEIGHT,BPP,SDL_SWSURFACE | (fullscreen=!fullscreen)?SDL_FULLSCREEN:0);
                              break;
                         case SDLK_ESCAPE:
                         case SDLK_q:
                         case SDLK_x:
                         case SDLK_F10:
                                quit=true;
                              break;
                         case SDLK_e:
                                Edit();
                              break;
                       }
                   break;
            }
      } 
    delete button_quit;
    delete button_play;
    delete button_options;
    delete button_edit;
    delete button_highs;
    delete button_continue;
    delete button_help;
    SDL_FreeSurface(menu);
    SDL_FreeSurface(text);
    SDL_FreeSurface(logo);
    TTF_CloseFont(font);
    TTF_Quit();
    SDL_Quit();
    return 0;
}
